from plain.assets.views import AssetView
from plain.http import Http404, ResponsePermanentRedirect, ResponseRedirect
from plain.utils.functional import cached_property
from plain.views import TemplateView, View

from .exceptions import PageNotFoundError, RedirectPageError
from .registry import registry


class PageViewMixin:
    @cached_property
    def page(self):
        # Passed manually by the kwargs in the path definition
        url_path = self.url_kwargs.get("url_path", "index")

        try:
            return registry.get_page(url_path)
        except PageNotFoundError:
            raise Http404()


class PageView(PageViewMixin, TemplateView):
    template_name = "page.html"

    def get_template_names(self) -> list[str]:
        """
        Allow for more specific user templates like
        markdown.html or html.html
        """
        return [self.page.get_template_name()] + super().get_template_names()

    def get_template_context(self):
        context = super().get_template_context()
        context["page"] = self.page
        self.page.set_template_context(context)  # Pass the standard context through
        return context


class PageRedirectView(PageViewMixin, View):
    def get(self):
        # Passed manually by the kwargs in the path definition
        url_path = self.url_kwargs.get("url_path", "index")

        url = self.page.vars.get("url")

        if not url:
            raise RedirectPageError(f"Redirect page {url_path} is missing a url")

        if self.page.vars.get("temporary", True):
            return ResponseRedirect(url)
        else:
            return ResponsePermanentRedirect(url)


class PageAssetView(PageViewMixin, AssetView):
    def get_url_path(self):
        return self.url_kwargs["url_path"]

    def get_asset_path(self, path):
        return self.page.absolute_path

    def get_debug_asset_path(self, path):
        return self.page.absolute_path



import os

from plain.urls import path

from .exceptions import PageNotFoundError
from .pages import Page


class PagesRegistry:
    """
    The registry loads up all the pages at once, so we only have to do a
    dict key lookup at runtime to get a page.
    """

    def __init__(self):
        # url path -> file path
        self.registered_pages = {}

    def get_page_urls(self):
        """
        Generate a list of real urls based on the files that exist.
        This way, you get a concrete url reversingerror if you try
        to refer to a page/url that isn't going to work.
        """
        paths = []

        for url_path in self.registered_pages.keys():
            if url_path == "":
                # The root index is a special case and should be
                # referred to as pages:index
                url = ""
                name = "index"
            else:
                url = url_path
                name = url_path

            page = self.get_page(url_path)
            view_class = page.get_view_class()

            paths.append(
                path(
                    url,
                    view_class,
                    name=name,
                    kwargs={"url_path": url_path},
                )
            )

        return paths

    def discover_pages(self, pages_dir):
        for root, _, files in os.walk(pages_dir):
            for file in files:
                relative_path = os.path.relpath(os.path.join(root, file), pages_dir)
                absolute_path = os.path.join(root, file)

                page_args = (relative_path, absolute_path)
                url_path = Page(*page_args).get_url_path()

                # Some pages don't get a url (like templates)
                if url_path is None:
                    continue

                self.registered_pages[url_path] = page_args

    def get_page(self, url_path):
        try:
            page_args = self.registered_pages[url_path]
            # Instantiate the page here, so we don't store a ton of cached data over time
            # as we render all the pages
            return Page(*page_args)
        except KeyError:
            raise PageNotFoundError(f"Could not find a page for {url_path}")


registry = PagesRegistry()



import os

from plain.packages import PackageConfig, packages
from plain.runtime import APP_PATH

from .registry import registry


class PlainPagesConfig(PackageConfig):
    name = "plain.pages"

    def ready(self):
        for pacakge_config in packages.get_package_configs():
            registry.discover_pages(
                os.path.join(pacakge_config.path, "templates", "pages")
            )

        registry.discover_pages(os.path.join(APP_PATH, "templates", "pages"))



from html.parser import HTMLParser

import mistune
from pygments import highlight
from pygments.formatters import html
from pygments.lexers import get_lexer_by_name

from plain.utils.text import slugify


class PagesRenderer(mistune.HTMLRenderer):
    def heading(self, text, level, **attrs):
        """Automatically add an ID to headings if one is not provided."""

        if "id" not in attrs:
            inner_text = get_inner_text(text)
            inner_text = inner_text.replace(
                ".", "-"
            )  # Replace dots with hyphens (slugify won't)
            attrs["id"] = slugify(inner_text)

        return super().heading(text, level, **attrs)

    def block_code(self, code, info=None):
        """Highlight code blocks using Pygments."""

        if info:
            lexer = get_lexer_by_name(info, stripall=True)
            formatter = html.HtmlFormatter(wrapcode=True)
            return highlight(code, lexer, formatter)

        return "<pre><code>" + mistune.escape(code) + "</code></pre>"


def render_markdown(content):
    renderer = PagesRenderer(escape=False)
    markdown = mistune.create_markdown(
        renderer=renderer, plugins=["strikethrough", "table"]
    )
    return markdown(content)


class InnerTextParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.text_content = []

    def handle_data(self, data):
        # Collect all text data
        self.text_content.append(data.strip())


def get_inner_text(html_content):
    parser = InnerTextParser()
    parser.feed(html_content)
    return " ".join([text for text in parser.text_content if text])
